@@ -76,10 +76,16 @@ function init(self)
 
         -- language
     ,   ["-ansi"]                   = ""
-    ,   ["-std=c99"]                = "-TP" -- compile as c++ files because msvc only support c89
-    ,   ["-std=c11"]                = "-TP" -- compile as c++ files because msvc only support c89
-    ,   ["-std=gnu99"]              = "-TP" -- compile as c++ files because msvc only support c89
-    ,   ["-std=gnu11"]              = "-TP" -- compile as c++ files because msvc only support c89
+    ,   ["-std=c99"]                = "-TP" -- compile as c++ files because msvc doesn't support c99
+    ,   ["-std=c11"]                = "-std:c11"
+    ,   ["-std=c17"]                = "-std:c17"
+    ,   ["-std=c2x"]                = "-std:clatest"
+    ,   ["-std=c23"]                = "-std:clatest"
+    ,   ["-std=gnu99"]              = "-TP" -- compile as c++ files because msvc doesn't support c99
+    ,   ["-std=gnu11"]              = "-std:c11"
+    ,   ["-std=gnu17"]              = "-std:c17"
+    ,   ["-std=gnu2x"]              = "-std:clatest"
+    ,   ["-std=gnu23"]              = "-std:clatest"
     ,   ["-std=.*"]                 = ""
 
         -- others
@@ -88,7 +94,7 @@ function init(self)
 end
 
 -- make the symbol flags
-function nf_symbols(self, levels, target)
+function nf_symbols(self, levels, opt)
     local flags = nil
     local values = hashset.from(levels)
     if values:has("debug") then
@@ -103,6 +109,7 @@ function nf_symbols(self, levels, target)
 
         -- generate *.pdb file
         local symbolfile = nil
+        local target = opt.target
         if target and target.symbolfile and not values:has("embed") then
             symbolfile = target:symbolfile()
         end
@@ -116,7 +123,7 @@ function nf_symbols(self, levels, target)
 
             -- check and add symbol output file
             --
-            -- @note we need use `{}` to wrap it to avoid expand it
+            -- @note we need to use `{}` to wrap it to avoid expand it
             -- https://github.com/xmake-io/xmake/issues/2061#issuecomment-1042590085
             local pdbflags = {"-Fd" .. (target:is_static() and symbolfile or path.join(symboldir, "compile." .. path.filename(symbolfile)))}
             if self:has_flags({"-FS", "-Fd" .. os.nuldev() .. ".pdb"}, "cxflags", { flagskey = "-FS -Fd" }) then
@@ -130,8 +137,7 @@ end
 
 -- make the fp-model flag
 function nf_fpmodel(self, level)
-    local maps =
-    {
+    local maps = {
         precise    = "-fp:precise" -- default
     ,   fast       = "-fp:fast"
     ,   strict     = "-fp:strict"
@@ -142,9 +148,16 @@ function nf_fpmodel(self, level)
 end
 
 -- make the warning flag
-function nf_warning(self, level)
-    local maps =
-    {
+function nf_warnings(self, levels)
+    local flags = {}
+    local values = hashset.from(levels)
+    if values:has("all") and values:has("extra") then
+        table.insert(flags, "-W4")
+        values:remove("all")
+        values:remove("extra")
+    end
+
+    local maps = {
         none       = "-w"
     ,   less       = "-W1"
     ,   more       = "-W3"
@@ -153,7 +166,15 @@ function nf_warning(self, level)
     ,   everything = "-Wall"
     ,   error      = "-WX"
     }
-    return maps[level]
+    for _, level in values:orderkeys() do
+        local flag = maps[level]
+        if flag then
+            table.insert(flags, flag)
+        end
+    end
+    if #flags > 0 then
+        return flags
+    end
 end
 
 -- make the optimize flag
@@ -162,17 +183,23 @@ function nf_optimize(self, level)
     {
         none        = "-Od"
     ,   faster      = "-Ox"
-    ,   fastest     = "-O2 -fp:fast"
-    ,   smallest    = "-O1 -GL" -- /GL and (/OPT:REF is on by default in linker), we need enable /ltcg
-    ,   aggressive  = "-O2 -fp:fast"
+    ,   fastest     = "-O2"
+    ,   smallest    = "-O1 -GL" -- /GL and (/OPT:REF is on by default in linker), we need to enable /ltcg
+    ,   aggressive  = "-O2"
     }
     return maps[level]
 end
 
--- make vs runtime flag
-function nf_runtime(self, vs_runtime)
-    if vs_runtime then
-        return "-" .. vs_runtime
+-- make the runtime flag
+function nf_runtime(self, runtime)
+    if runtime then
+        local maps = {
+            MT = "-MT",
+            MD = "-MD",
+            MTd = "-MTd",
+            MDd = "-MDd"
+        }
+        return maps[runtime]
     end
 end
 
@@ -180,11 +207,14 @@ end
 function nf_vectorext(self, extension)
     local maps =
     {
-        sse    = "-arch:SSE"
-    ,   sse2   = "-arch:SSE2"
-    ,   avx    = "-arch:AVX"
-    ,   avx2   = "-arch:AVX2"
-    ,   fma    = "-arch:AVX2"
+        sse        = "-arch:SSE"
+    ,   sse2       = "-arch:SSE2"
+    ,   ["sse4.2"] = "/d2archSSE42" -- the only undocumented way works, @see https://github.com/xmake-io/xmake/issues/3786
+    ,   avx        = "-arch:AVX"
+    ,   avx2       = "-arch:AVX2"
+    ,   avx512     = "-arch:AVX512" -- for msvc 2019+ avx512 support
+    ,   fma        = "-arch:AVX2"
+    ,   all        = {"-arch:SSE", "-arch:SSE2", "/d2archSSE42", "-arch:AVX", "-arch:AVX2", "-arch:AVX512"}
     }
     local flag = maps[extension]
     if flag and self:has_flags(flag, "cxflags") then
@@ -203,12 +233,16 @@ function nf_language(self, stdname)
             -- stdc
             c99       = "-TP" -- compile as c++ files because older msvc only support c89
         ,   gnu99     = "-TP"
-        ,   c11       = {"-std:c11", "-TP"}
-        ,   gnu11     = {"-std:c11", "-TP"}
-        ,   c17       = {"-std:c17", "-TP"}
-        ,   gnu17     = {"-std:c17", "-TP"}
-        ,   clatest   = {"-std:c17", "-std:c11"}
-        ,   gnulatest = {"-std:c17", "-std:c11"}
+        ,   c11       = {"-std:c11", "-std:clatest", "-TP"}
+        ,   gnu11     = {"-std:c11", "-std:clatest", "-TP"}
+        ,   c17       = {"-std:c17", "-std:clatest", "-TP"}
+        ,   gnu17     = {"-std:c17", "-std:clatest", "-TP"}
+        ,   c2x       = {"-std:c23", "-std:clatest", "-TP"}
+        ,   gnu2x     = {"-std:c23", "-std:clatest", "-TP"}
+        ,   c23       = {"-std:c23", "-std:clatest", "-TP"}
+        ,   gnu23     = {"-std:c23", "-std:clatest", "-TP"}
+        ,   clatest   = {"-std:clatest", "-std:c23", "-std:c17", "-std:c11", "-TP"}
+        ,   gnulatest   = {"-std:clatest", "-std:c23", "-std:c17", "-std:c11", "-TP"}
         }
     end
 
@@ -232,6 +266,8 @@ function nf_language(self, stdname)
         ,   gnuxx23     = {"-std:c++23", "-std:c++latest"}
         ,   cxx2b       = {"-std:c++23", "-std:c++latest"}
         ,   gnuxx2b     = {"-std:c++23", "-std:c++latest"}
+        ,   cxx26       = {"-std:c++26", "-std:c++latest"}
+        ,   gnuxx26     = {"-std:c++26", "-std:c++latest"}
         ,   cxxlatest   = "-std:c++latest"
         ,   gnuxxlatest = "-std:c++latest"
         }
@@ -278,6 +314,15 @@ function nf_includedir(self, dir)
     return {"-I" .. path.translate(dir)}
 end
 
+-- make the force include flag
+function nf_forceinclude(self, headerfile, opt)
+    local target = opt.target
+    local sourcekinds = target and target:extraconf("forceincludes", headerfile, "sourcekinds")
+    if not sourcekinds or table.contains(table.wrap(sourcekinds), self:kind()) then
+        return {"-FI", headerfile}
+    end
+end
+
 -- make the sysincludedir flag
 function nf_sysincludedir(self, dir)
     local has_external_includedir = _g._HAS_EXTERNAL_INCLUDEDIR
@@ -312,25 +357,67 @@ function nf_exception(self, exp)
     return maps[exp]
 end
 
+-- make the encoding flag
+-- @see https://github.com/xmake-io/xmake/issues/2471
+--
+-- e.g.
+-- set_encodings("utf-8")
+-- set_encodings("source:utf-8", "target:utf-8")
+function nf_encoding(self, encoding)
+    local kind
+    local charset
+    local splitinfo = encoding:split(":")
+    if #splitinfo > 1 then
+        kind = splitinfo[1]
+        charset = splitinfo[2]
+    else
+        charset = encoding
+    end
+    local charsets = {
+        ["utf-8"] = "utf-8",
+        utf8 = "utf-8",
+        gb2312 = "gb2312"
+    }
+    local flags = {}
+    charset = charsets[charset:lower()]
+    if charset then
+        if not kind and charset == "utf-8" then
+            table.insert(flags, "/utf-8")
+        else
+            if kind == "source" or not kind then
+                table.insert(flags, "-source-charset:" .. charset)
+            end
+            if kind == "target" or not kind then
+                table.insert(flags, "-execution-charset:" .. charset)
+            end
+        end
+    end
+    if #flags > 0 then
+        return flags
+    end
+end
+
 -- make the c precompiled header flag
-function nf_pcheader(self, pcheaderfile, target)
+function nf_pcheader(self, pcheaderfile, opt)
     if self:kind() == "cc" then
+        local target = opt.target
         local objectfiles = target:objectfiles()
         if objectfiles then
             table.insert(objectfiles, target:pcoutputfile("c") .. ".obj")
         end
-        return {"-Yu" .. path.filename(pcheaderfile), "-FI" .. path.filename(pcheaderfile), "-Fp" .. target:pcoutputfile("c")}
+        return {"-Yu" .. path.absolute(pcheaderfile), "-FI" .. path.absolute(pcheaderfile), "-Fp" .. target:pcoutputfile("c")}
     end
 end
 
 -- make the c++ precompiled header flag
-function nf_pcxxheader(self, pcheaderfile, target)
+function nf_pcxxheader(self, pcheaderfile, opt)
     if self:kind() == "cxx" then
+        local target = opt.target
         local objectfiles = target:objectfiles()
         if objectfiles then
             table.insert(objectfiles, target:pcoutputfile("cxx") .. ".obj")
         end
-        return {"-Yu" .. path.filename(pcheaderfile), "-FI" .. path.filename(pcheaderfile), "-Fp" .. target:pcoutputfile("cxx")}
+        return {"-Yu" .. path.absolute(pcheaderfile), "-FI" .. path.absolute(pcheaderfile), "-Fp" .. target:pcoutputfile("cxx")}
     end
 end
 
@@ -521,7 +608,7 @@ end
 -- compile preprocessed file
 function _compile_preprocessed_file(program, cppinfo, opt)
     local outdata, errdata = vstool.iorunv(program, winos.cmdargv(table.join(cppinfo.cppflags, "-Fo" .. cppinfo.objectfile, cppinfo.cppfile)), opt)
-    -- we need get warning information from output
+    -- we need to get warning information from output
     cppinfo.outdata = outdata
     cppinfo.errdata = errdata
 end
@@ -592,7 +679,7 @@ function compile(self, sourcefile, objectfile, dependinfo, flags, opt)
                 end
             end
 
-            -- we need show full file path to goto error position if xmake is called in vstudio
+            -- we need to show full file path to goto error position if xmake is called in vstudio
             -- https://github.com/xmake-io/xmake/issues/1049
             if _is_in_vstudio() then
                 if compflags == flags then
@@ -646,7 +733,7 @@ function compile(self, sourcefile, objectfile, dependinfo, flags, opt)
             function (ok, outdata, errdata)
 
                 -- show warnings?
-                if ok and policy.build_warnings() then
+                if ok and policy.build_warnings(opt) then
                     local output = outdata or ""
                     if #output:trim() == 0 then
                         output = errdata or ""
