@@ -81,12 +81,12 @@ function load(self)
 end
 
 -- make the strip flag
-function nf_strip(self, level, target)
+function nf_strip(self, level)
     local maps = {
         debug = "-Wl,-S"
     ,   all   = "-s"
     }
-    if self:is_plat("macosx", "iphoneos") then
+    if self:is_plat("macosx", "iphoneos", "watchos", "appletvos", "applexros") then
         maps.all = {"-Wl,-x", "-Wl,-dead_strip"}
     elseif self:is_plat("windows") then
         -- clang does not it on windows, TODO maybe we need test it for gcc
@@ -182,9 +182,45 @@ function nf_vectorext(self, extension)
     ,   neon       = "-mfpu=neon"
     ,   all        = "-march=native"
     }
+    if extension == "all" and self:is_cross() then
+        -- https://github.com/xmake-io/xmake-repo/pull/4040#discussion_r1605121207
+        maps[extension] = nil
+    end
     return maps[extension]
 end
 
+-- has -static-libstdc++?
+function _has_static_libstdcxx(self)
+    local has_static_libstdcxx = _g._HAS_STATIC_LIBSTDCXX
+    if has_static_libstdcxx == nil then
+        if self:has_flags("-static-libstdc++ -Werror", "ldflags", {flagskey = "gcc_static_libstdcxx"}) then
+            has_static_libstdcxx = true
+        end
+        has_static_libstdcxx = has_static_libstdcxx or false
+        _g._HAS_STATIC_LIBSTDCXX = has_static_libstdcxx
+    end
+    return has_static_libstdcxx
+end
+
+-- make the runtime flag
+function nf_runtime(self, runtime, opt)
+    opt = opt or {}
+    local maps
+    local kind = self:kind()
+    if not self:is_plat("android") then -- we will set runtimes in android ndk toolchain
+        maps = maps or {}
+        if kind == "ld" or kind == "sh" then
+            local target = opt.target
+            if target and target.sourcekinds and table.contains(table.wrap(target:sourcekinds()), "cxx") then
+                if runtime:endswith("_static") and _has_static_libstdcxx(self) then
+                    maps["stdc++_static"] = "-static-libstdc++"
+                end
+            end
+        end
+    end
+    return maps and maps[runtime]
+end
+
 -- make the language flag
 function nf_language(self, stdname)
 
@@ -203,8 +239,10 @@ function nf_language(self, stdname)
         ,   gnu11       = "-std=gnu11"
         ,   c17         = "-std=c17"
         ,   gnu17       = "-std=gnu17"
-        ,   clatest     = {"-std=c2x", "-std=c17", "-std=c11", "-std=c99", "-std=c89", "-ansi"}
-        ,   gnulatest   = {"-std=c2x", "-std=gnu17", "-std=gnu11", "-std=gnu99", "-std=gnu89", "-ansi"}
+        ,   c23         = {"-std=c23", "-std=c2x"}
+        ,   gnu23       = {"-std=gnu23", "-std=gnu2x"}
+        ,   clatest     = {"-std=c23", "-std=c2x", "-std=c17", "-std=c11", "-std=c99", "-std=c89", "-ansi"}
+        ,   gnulatest   = {"-std=gnu23", "-std=gnu2x", "-std=gnu17", "-std=gnu11", "-std=gnu99", "-std=gnu89", "-ansi"}
         }
     end
 
@@ -231,8 +269,12 @@ function nf_language(self, stdname)
         ,   gnuxx23      = {"-std=gnu++23", "-std=c++2b"}
         ,   cxx2b        = "-std=c++2b"
         ,   gnuxx2b      = "-std=gnu++2b"
-        ,   cxxlatest    = {"-std=c++23", "-std=c++2b", "-std=c++20", "-std=c++2a", "-std=c++17", "-std=c++14", "-std=c++11", "-std=c++1z", "-std=c++98"}
-        ,   gnuxxlatest  = {"-std=gnu++23", "-std=gnu++2b", "-std=gnu++20", "-std=gnu++2a", "-std=gnu++17", "-std=gnu++14", "-std=gnu++11", "-std=c++1z", "-std=gnu++98"}
+        ,   cxx2c        = "-std=c++2c"
+        ,   gnuxx2c      = "-std=gnu++2c"
+        ,   cxx26        = {"-std=c++26", "-std=c++2c"}
+        ,   gnuxx26      = {"-std=gnu++26", "-std=gnu++2c"}
+        ,   cxxlatest    = {"-std=c++26", "-std=c++2c", "-std=c++23", "-std=c++2b", "-std=c++20", "-std=c++2a", "-std=c++17", "-std=c++14", "-std=c++11", "-std=c++1z", "-std=c++98"}
+        ,   gnuxxlatest  = {"-std=gnu++26", "-std=gnu++2c", "-std=gnu++23", "-std=gnu++2", "-std=gnu++20", "-std=gnu++2a", "-std=gnu++17", "-std=gnu++14", "-std=gnu++11", "-std=c++1z", "-std=gnu++98"}
         }
         local cxxmaps2 = {}
         for k, v in pairs(_g.cxxmaps) do
@@ -283,7 +325,8 @@ function nf_sysincludedir(self, dir)
 end
 
 -- make the force include flag
-function nf_forceinclude(self, headerfile, target)
+function nf_forceinclude(self, headerfile, opt)
+    local target = opt.target
     local sourcekinds = target and target:extraconf("forceincludes", headerfile, "sourcekinds")
     if not sourcekinds or table.contains(table.wrap(sourcekinds), self:kind()) then
         return {"-include", headerfile}
@@ -292,7 +335,9 @@ end
 
 -- make the link flag
 function nf_link(self, lib)
-    if lib:endswith(".a") or lib:endswith(".so") or lib:endswith(".dylib") or lib:endswith(".lib") then
+    if self:is_plat("linux") and (lib:endswith(".a") or lib:endswith(".so")) and not lib:find(path.sep(), 1, true) then
+        return "-l:" .. lib
+    elseif lib:endswith(".a") or lib:endswith(".so") or lib:endswith(".dylib") or lib:endswith(".lib") then
         return lib
     else
         return "-l" .. lib
@@ -304,19 +349,63 @@ function nf_syslink(self, lib)
     return nf_link(self, lib)
 end
 
+-- make the link group flag
+function nf_linkgroup(self, linkgroup, opt)
+    local linkflags = {}
+    for _, lib in ipairs(linkgroup) do
+        table.insert(linkflags, nf_link(self, lib))
+    end
+    local flags = {}
+    local extra = opt.extra
+    if extra and not self:is_plat("macosx", "windows", "mingw") then
+        local group = extra.group
+        local whole = extra.whole
+        if group and whole then
+            -- https://github.com/xmake-io/xmake/issues/4308
+            table.join2(flags, "-Wl,--whole-archive", "-Wl,--start-group", linkflags, "-Wl,--end-group", "-Wl,--no-whole-archive")
+        elseif group then
+            table.join2(flags, "-Wl,--start-group", linkflags, "-Wl,--end-group")
+        elseif whole then
+            table.join2(flags, "-Wl,--whole-archive", linkflags, "-Wl,--no-whole-archive")
+        end
+        local static = extra.static
+        if static then
+            table.join2(flags, "-Wl,-Bstatic", linkflags, "-Wl,-Bdynamic")
+        end
+    end
+    if #flags == 0 then
+        flags = linkflags
+    end
+    return flags
+end
+
 -- make the linkdir flag
 function nf_linkdir(self, dir)
     return {"-L" .. path.translate(dir)}
 end
 
 -- make the rpathdir flag
-function nf_rpathdir(self, dir)
+function nf_rpathdir(self, dir, opt)
+    opt = opt or {}
+    local extra = opt.extra
+    if extra and extra.installonly then
+        return
+    end
     dir = path.translate(dir)
     if self:has_flags("-Wl,-rpath=" .. dir, "ldflags") then
         local flags = {"-Wl,-rpath=" .. (dir:gsub("@[%w_]+", function (name)
             local maps = {["@loader_path"] = "$ORIGIN", ["@executable_path"] = "$ORIGIN"}
             return maps[name]
         end))}
+        -- add_rpathdirs("...", {runpath = false})
+        -- https://github.com/xmake-io/xmake/issues/5109
+        if extra then
+            if extra.runpath == false and self:has_flags("-Wl,-rpath=" .. dir .. ",--disable-new-dtags", "ldflags") then
+                flags[1] = flags[1] .. ",--disable-new-dtags"
+            elseif extra.runpath == true and self:has_flags("-Wl,-rpath=" .. dir .. ",--enable-new-dtags", "ldflags") then
+                flags[1] = flags[1] .. ",--enable-new-dtags"
+            end
+        end
         if self:is_plat("bsd") then
             -- FreeBSD ld must have "-zorigin" with "-rpath".  Otherwise, $ORIGIN is not translated and it is literal.
             table.insert(flags, 1, "-Wl,-zorigin")
@@ -385,8 +474,9 @@ function nf_encoding(self, encoding)
 end
 
 -- make the c precompiled header flag
-function nf_pcheader(self, pcheaderfile, target)
+function nf_pcheader(self, pcheaderfile, opt)
     if self:kind() == "cc" then
+        local target = opt.target
         local pcoutputfile = target:pcoutputfile("c")
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
@@ -397,8 +487,9 @@ function nf_pcheader(self, pcheaderfile, target)
 end
 
 -- make the c++ precompiled header flag
-function nf_pcxxheader(self, pcheaderfile, target)
+function nf_pcxxheader(self, pcheaderfile, opt)
     if self:kind() == "cxx" then
+        local target = opt.target
         local pcoutputfile = target:pcoutputfile("cxx")
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
@@ -409,8 +500,9 @@ function nf_pcxxheader(self, pcheaderfile, target)
 end
 
 -- make the objc precompiled header flag
-function nf_pmheader(self, pcheaderfile, target)
+function nf_pmheader(self, pcheaderfile, opt)
     if self:kind() == "mm" then
+        local target = opt.target
         local pcoutputfile = target:pcoutputfile("m")
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
@@ -421,8 +513,9 @@ function nf_pmheader(self, pcheaderfile, target)
 end
 
 -- make the objc++ precompiled header flag
-function nf_pmxxheader(self, pcheaderfile, target)
+function nf_pmxxheader(self, pcheaderfile, opt)
     if self:kind() == "mxx" then
+        local target = opt.target
         local pcoutputfile = target:pcoutputfile("mxx")
         if self:name() == "clang" then
             return {"-include", pcheaderfile, "-include-pch", pcoutputfile}
@@ -457,8 +550,10 @@ function linkargv(self, objectfiles, targetkind, targetfile, flags, opt)
     -- add rpath for dylib (macho), e.g. -install_name @rpath/file.dylib
     local flags_extra = {}
     if targetkind == "shared" and self:is_plat("macosx", "iphoneos", "watchos") then
-        table.insert(flags_extra, "-install_name")
-        table.insert(flags_extra, "@rpath/" .. path.filename(targetfile))
+        if not table.contains(flags, "-install_name") then
+            table.insert(flags_extra, "-install_name")
+            table.insert(flags_extra, "@rpath/" .. path.filename(targetfile))
+        end
     end
 
     -- add `-Wl,--out-implib,outputdir/libxxx.a` for xxx.dll on mingw/gcc
@@ -480,13 +575,14 @@ end
 -- maybe we need to use os.vrunv() to show link output when enable verbose information
 -- @see https://github.com/xmake-io/xmake/discussions/2916
 --
-function link(self, objectfiles, targetkind, targetfile, flags)
+function link(self, objectfiles, targetkind, targetfile, flags, opt)
+    opt = opt or {}
     os.mkdir(path.directory(targetfile))
     local program, argv = linkargv(self, objectfiles, targetkind, targetfile, flags)
     if option.get("verbose") then
-        os.execv(program, argv, {envs = self:runenvs()})
+        os.execv(program, argv, {envs = self:runenvs(), shell = opt.shell})
     else
-        os.vrunv(program, argv, {envs = self:runenvs()})
+        os.vrunv(program, argv, {envs = self:runenvs(), shell = opt.shell})
     end
 end
 
@@ -504,6 +600,12 @@ function _has_color_diagnostics(self)
                 elseif self:has_flags("-fcolor-diagnostics", "cxflags") then
                     colors_diagnostics = "-fcolor-diagnostics"
                 end
+
+                -- enable color output for windows, @see https://github.com/xmake-io/xmake-vscode/discussions/260
+                if colors_diagnostics and self:name() == "clang" and is_host("windows") and
+                    self:has_flags("-fansi-escape-codes", "cxflags") then
+                    colors_diagnostics = table.join(colors_diagnostics, "-fansi-escape-codes")
+                end
             end
         end
         colors_diagnostics = colors_diagnostics or false
@@ -640,6 +742,9 @@ function _preprocess(program, argv, opt)
 
     -- do preprocess
     local cppinfo = try {function ()
+        if is_host("windows") then
+            cppflags = winos.cmdargv(cppflags, {escape = true})
+        end
         local outdata, errdata = os.iorunv(program, cppflags, opt)
         return {outdata = outdata, errdata = errdata,
                 sourcefile = sourcefile, objectfile = objectfile, cppfile = cppfile, cppflags = flags}
@@ -655,7 +760,11 @@ end
 
 -- compile preprocessed file
 function _compile_preprocessed_file(program, cppinfo, opt)
-    local outdata, errdata = os.iorunv(program, table.join(cppinfo.cppflags, "-o", cppinfo.objectfile, cppinfo.cppfile), opt)
+    local argv = table.join(cppinfo.cppflags, "-o", cppinfo.objectfile, cppinfo.cppfile)
+    if is_host("windows") then
+        argv = winos.cmdargv(argv, {escape = true})
+    end
+    local outdata, errdata = os.iorunv(program, argv, opt)
     -- we need to get warning information from output
     cppinfo.outdata = outdata
     cppinfo.errdata = errdata
@@ -664,19 +773,23 @@ end
 -- do compile
 function _compile(self, sourcefile, objectfile, compflags, opt)
     opt = opt or {}
-    local program, argv = compargv(self, sourcefile, objectfile, compflags)
+    local program, argv = compargv(self, sourcefile, objectfile, compflags, opt)
     local function _compile_fallback()
-        return os.iorunv(program, argv, {envs = self:runenvs()})
+        local runargv = argv
+        if is_host("windows") then
+            runargv = winos.cmdargv(argv, {escape = true})
+        end
+        return os.iorunv(program, runargv, {envs = self:runenvs(), shell = opt.shell})
     end
     local cppinfo
     if distcc_build_client.is_distccjob() and distcc_build_client.singleton():has_freejobs() then
         cppinfo = distcc_build_client.singleton():compile(program, argv, {envs = self:runenvs(),
             preprocess = _preprocess, compile = _compile_preprocessed_file, compile_fallback = _compile_fallback,
-            tool = self, remote = true})
+            tool = self, remote = true, shell = opt.shell})
     elseif build_cache.is_enabled(opt.target) and build_cache.is_supported(self:kind()) then
         cppinfo = build_cache.build(program, argv, {envs = self:runenvs(),
             preprocess = _preprocess, compile = _compile_preprocessed_file, compile_fallback = _compile_fallback,
-            tool = self})
+            tool = self, shell = opt.shell})
     end
     if cppinfo then
         return cppinfo.outdata, cppinfo.errdata
@@ -686,13 +799,13 @@ function _compile(self, sourcefile, objectfile, compflags, opt)
 end
 
 -- make the compile arguments list for the precompiled header
-function _compargv_pch(self, pcheaderfile, pcoutputfile, flags)
+function _compargv_pch(self, pcheaderfile, pcoutputfile, flags, opt)
 
     -- remove "-include xxx.h" and "-include-pch xxx.pch"
     local pchflags = {}
     local include = false
     for _, flag in ipairs(flags) do
-        if not flag:find("-include", 1, true) then
+        if not flag:startswith("-include") then
             if not include then
                 table.insert(pchflags, flag)
             end
@@ -718,17 +831,21 @@ function _compargv_pch(self, pcheaderfile, pcoutputfile, flags)
     end
 
     -- make the compile arguments list
-    return self:program(), table.join("-c", pchflags, "-o", pcoutputfile, pcheaderfile)
+    local argv = table.join("-c", pchflags, "-o", pcoutputfile, pcheaderfile)
+    return self:program(), argv
 end
 
 -- make the compile arguments list
-function compargv(self, sourcefile, objectfile, flags)
+function compargv(self, sourcefile, objectfile, flags, opt)
+
     -- precompiled header?
     local extension = path.extension(sourcefile)
     if (extension:startswith(".h") or extension == ".inl") then
-        return _compargv_pch(self, sourcefile, objectfile, flags)
+        return _compargv_pch(self, sourcefile, objectfile, flags, opt)
     end
-    return self:program(), table.join("-c", flags, "-o", objectfile, sourcefile)
+
+    local argv = table.join("-c", flags, "-o", objectfile, sourcefile)
+    return self:program(), argv
 end
 
 -- compile the source file
@@ -806,7 +923,7 @@ function compile(self, sourcefile, objectfile, dependinfo, flags, opt)
         {
             function (ok, outdata, errdata)
                 -- show warnings?
-                if ok and errdata and #errdata > 0 and policy.build_warnings() then
+                if ok and errdata and #errdata > 0 and policy.build_warnings(opt) then
                     local lines = errdata:split('\n', {plain = true})
                     if #lines > 0 then
                         if not option.get("diagnosis") then
